/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.servlet.RegistryRealmFactory;
import org.wso2.usermanager.Realm;

import java.util.*;

public class JDBCRegistryTest extends TestCase {

    /**
     * Registry instance for use in tests. Note that there should be only one Registry instance
     * in a JVM.
     */
    protected static Registry registry = null;

    public void setUp() {
        try {
            if (registry == null) {
                Realm registryRealm =
                        RegistryRealmFactory.createInMemoryRegistryRealm();
                JDBCRegistry jdbcRegistry = new JDBCRegistry();
                jdbcRegistry.init(registryRealm);
                registry = jdbcRegistry;
            }
        } catch (RegistryException e) {
            fail("Failed to initialize the registry.");
        }
    }

    public void testFlatResourceHandling() {
        Resource r1 = new Resource();
        r1.setAuthorUserName("Chathura");
        r1.setDescription("This is a test resource used for registry testing.");
        String r1Content = "<c>This is r1 content</c>";
        r1.setContent(r1Content.getBytes());
        r1.setProperty("p1", "v1");

        try {
            registry.put("/r1", r1);
        } catch (RegistryException e) {
            fail("Couldn't put content resource /r1");
        }

        Resource r1f = null;
        try {
            r1f = registry.get("/r1");
        } catch (RegistryException e) {
            fail("Couldn't get content resource /r1");
        }

        assertEquals("Content is not equal.",
                new String((byte[]) r1.getContent()), new String((byte[]) r1f.getContent()));

        assertEquals("Author is not equal.", r1.getAuthorUserName(), r1f.getAuthorUserName());

        assertEquals("Description is not equal.", r1.getDescription(), r1f.getDescription());

        assertEquals("Property p1 should contain the value v1", r1f.getProperty("p1"), "v1");

        try {
            registry.delete("/r1");
        } catch (RegistryException e) {
            fail("Couldn't delete content resource /r1");
        }


        Resource newR1 = null;

        boolean failed = false;
        try {
            newR1 = registry.get("/r1");
        } catch (RegistryException e) {
            failed = true;
        }

        assertTrue("Deleted resource /r1 is returned on get.", failed);
    }

    public void testHierarchicalResourceHandling() {

        // add a resource

        Resource r1 = new Resource();
        r1.setAuthorUserName("Author R1");
        byte[] r1content = "R1 content".getBytes();
        r1.setContent(r1content);

        try {
            registry.put("/d1/r1", r1);
        } catch (RegistryException e) {
            fail("Valid put resource scenario failed.");
        }

        Resource d1 = null;
        try {
            d1 = registry.get("/d1");
        } catch (RegistryException e) {
            fail("parent directory was not cerated.");
        }

        assertTrue("/d1 should be a collection.", d1.isDirectory());

        assertTrue("Content of /d1 should be a String[]", d1.getContent() instanceof String[]);

        String[] children = (String[]) d1.getContent();
        boolean found = false;
        for (int i = 0; i < children.length; i++) {
            if (children[i].startsWith("/d1/r1")) {
                found = true;
                break;
            }
        }
        assertTrue("/d1/r1 should be a child of /d1", found);

        Resource r1f = null;
        try {
            r1f = registry.get("/d1/r1");
        } catch (RegistryException e) {
            fail("resource was not stored.");
        }

        assertEquals("Resource content is not stored correctly.",
                new String((byte[]) r1.getContent()), new String((byte[]) r1f.getContent()));

        try {
            registry.delete("/d1");
        } catch (RegistryException e) {
            fail("Valid resource delete scenario failed.");
        }

        boolean f1 = false;
        try {
            registry.get("/d1");
        } catch (RegistryException e) {
            f1 = true;
        }
        assertTrue("Deleted collection /d1 is not marked as deleted.", f1);

        Resource newD1R1 = null;
        boolean f2 = false;
        try {
            newD1R1 = registry.get("/d1/r1");
        } catch (RegistryException e) {
            f2 = true;
        }
        assertTrue("Deleted collection /d1/r1 is not marked as deleted.", f2);
    }

    public void testResourceVersioning() {
        Resource r1 = new Resource();
        r1.setAuthorUserName("writer1");
        byte[] r1Content = "R1 content".getBytes();
        r1.setContent(r1Content);

        try {
            registry.put("/r5", r1);
        } catch (RegistryException e) {
            fail("Valid put failed.");
        }

        // first update
        Resource redit1 = null;
        try {
            redit1 = registry.get("/r5");
        } catch (RegistryException e) {
            fail("Valid get failed.");
        }

        byte[] newR1Content = "New content".getBytes();
        redit1.setContent(newR1Content);

        try {
            registry.put("/r5", redit1);
        } catch (RegistryException e) {
            fail("Vaild put failed.");
        }

        // second update
        Resource redit2 = null;
        try {
            redit2 = registry.get("/r5");
        } catch (RegistryException e) {
            fail("Valid get failed.");
        }

        byte[] newR1Content2 = "New content2".getBytes();
        redit2.setContent(newR1Content2);

        try {
            registry.put("/r5", redit2);
        } catch (RegistryException e) {
            fail("Vaild put failed.");
        }

        String[] versionPaths = null;
        try {
            versionPaths = registry.getVersions("/r5");
        } catch (RegistryException e) {
            fail("Failed to get version paths.");
        }

        Resource v1 = null;
        Resource v2 = null;
        Resource v3 = null;
        try {
            v1 = registry.get(versionPaths[0]);
            v2 = registry.get(versionPaths[1]);
            v3 = registry.get(versionPaths[2]);
        } catch (RegistryException e) {
            e.printStackTrace();
            fail("Failed to get versioned resource.");
        }

        String content1 = new String((byte[]) v1.getContent());
        String content2 = new String((byte[]) v2.getContent());
        String content3 = new String((byte[]) v3.getContent());

        assertEquals("Content is not versioned properly.", content1, "R1 content");
        assertEquals("Content is not versioned properly.", content2, "New content");
        assertEquals("Content is not versioned properly.", content3, "New content2");

        try {
            registry.restoreVersion(versionPaths[0]);
        } catch (RegistryException e) {
            fail("Valid restore version failed.");
        }

        Resource r5restored = null;
        try {
            r5restored = registry.get("/r5");
        } catch (RegistryException e) {
            fail("Valid get failed.");
        }

        String restoredContent = new String((byte[]) r5restored.getContent());
        assertEquals("Content is not restored properly.", "R1 content", restoredContent);
    }

    //public void testDeleteWithVersioning() {
    //
    //    String r1Content = "r1 content1";
    //    Resource r1 = new Resource();
    //    r1.setAuthorUserName("Chathura");
    //    r1.setContent(r1Content.getBytes());
    //
    //    try {
    //        registry.put("/c12/r1", r1);
    //    } catch (RegistryException e) {
    //        fail("Valid put failed.");
    //    }
    //
    //    try {
    //        registry.delete("/c12/r1");
    //    } catch (RegistryException e) {
    //        fail("Valid delete failed.");
    //    }
    //
    //    String r2Content = "r2 content1";
    //    Resource r2 = new Resource();
    //    r2.setAuthorUserName("Samitha");
    //    r2.setContent(r2Content.getBytes());
    //
    //    try {
    //        registry.put("/c12/r1", r2);
    //    } catch (RegistryException e) {
    //        fail("Valid put failed.");
    //    }
    //
    //    int a = 1;
    //}

    public void testCollectionVersioning() {
        String r1Content = "r1 content1";
        Resource r1 = new Resource();
        r1.setAuthorUserName("Chathura");
        r1.setContent(r1Content.getBytes());

        try {
            registry.put("/c10/r1", r1);
        } catch (RegistryException e) {
            fail("Valid put failed.");
        }

        String r2Content = "r2 content2";
        Resource r2 = new Resource();
        r2.setAuthorUserName("Deepal");
        r2.setContent(r2Content.getBytes());

        try {
            registry.put("/c10/r2", r2);
        } catch (RegistryException e) {
            fail("Valid put failed.");
        }

        String[] versionPaths = null;
        try {
            versionPaths = registry.getVersions("/c10");
        } catch (RegistryException e) {
            fail("Valid get versions failed.");
        }

        Resource c10v1 = null;
        Resource c10v2 = null;
        try {
            c10v1 = registry.get(versionPaths[0]);
            c10v2 = registry.get(versionPaths[1]);
        } catch (RegistryException e) {
            fail("Valid get failed.");
        }

        String[] chidrenOfv1 = (String[]) c10v1.getContent();
        assertTrue("collection content is not versioned properly.", containsString(chidrenOfv1, "/c10/r1"));

        String[] chidrenOfv2 = (String[]) c10v2.getContent();
        assertTrue("collection content is not versioned properly.", containsString(chidrenOfv2, "/c10/r1"));
        assertTrue("collection content is not versioned properly.", containsString(chidrenOfv2, "/c10/r2"));

        try {
            registry.restoreVersion(versionPaths[0]);
        } catch (RegistryException e) {
            fail("Valid restore failed.");
        }

        Resource restoredC10 = null;
        try {
            restoredC10 = registry.get("/c10");
        } catch (RegistryException e) {
            fail("Valid get failed.");
        }

        String[] restoredC10Children = (String[]) restoredC10.getContent();
        assertTrue("Collection children are not restored properly.", containsString(restoredC10Children, "/c10/r1"));
        assertTrue("Collection children are not restored properly.", !containsString(restoredC10Children, "/c10/r2"));
    }

    

    public void testComments() {
        // add a resource

        try {
            Resource r1 = new Resource();
            r1.setAuthorUserName("Author R1");
            byte[] r1content = "R1 content".getBytes();
            r1.setContent(r1content);
            registry.put("/d12/r1", r1);

        } catch (RegistryException e) {
            fail("Failed to put test resources.");
        }

        String comment1 = "this can be used as a test resource.";
        String comment2 = "I like this";
        try {
            registry.addComment("/d12/r1", new Comment(comment1));
            registry.addComment("/d12/r1", new Comment(comment2));
        } catch (RegistryException e) {
            fail("Valid tagging scenario failed");
        }

        Comment[] comments = null;
        try {
            comments = registry.getComments("/d12/r1");
        } catch (RegistryException e) {
            fail("Failed to get comments for the resource /d12/r1");
        }

        boolean commentFound = false;
        for (int i = 0; i < comments.length; i++) {
            if (comments[i].getCommentText().equals(comment1)) {
                commentFound = true;
                break;
            }
        }
        assertTrue("comment '" + comment1 +
                "' is not associated with the artifact /d12/r1", commentFound);

        try {
            Resource commentsResource = registry.get("/d12/r1?comments");
            assertTrue("Comment collection resource should be a directory.",
                    commentsResource.isDirectory());
            String[] commentPaths = (String[]) commentsResource.getContent();

            List commentTexts = new ArrayList();
            for (int i = 0; i < commentPaths.length; i++) {
                Resource commentResource = registry.get(commentPaths[i]);
                commentTexts.add(commentResource.getContent());
            }

            assertTrue(comment1 + " is not associated for resource /d12/r1.", commentTexts.contains(comment1));
            assertTrue(comment2 + " is not associated for resource /d12/r1.", commentTexts.contains(comment2));

        } catch (RegistryException e) {
            e.printStackTrace();
            fail("Failed to get comments form URL: /d12/r1?comments");
        }

        try {
            registry.delete("/d12");
        } catch (RegistryException e) {
            fail("Failed to delete test resources.");
        }
    }

    public void testRatings() {
        // add a resource

        try {
            Resource r1 = new Resource();
            r1.setAuthorUserName("Author R1");
            byte[] r1content = "R1 content".getBytes();
            r1.setContent(r1content);
            registry.put("/d13/r1", r1);

        } catch (RegistryException e) {
            fail("Failed to put test resources.");
        }

        try {
            registry.rateResource("/d13/r1", 4);
        } catch (RegistryException e) {
            fail("Couldn't rate the resource /d13/r1");
        }

        float rating = 0;
        try {
            rating = registry.getAverageRating("/d13/r1");
        } catch (RegistryException e) {
            fail("Couldn't get the rating of the resource /d13/r1");
        }

        assertEquals("Rating of the resource /d13/r1 should be 4.", rating, (float) 4.0, (float) 0.01);

        try {
            registry.delete("/d13");
        } catch (RegistryException e) {
            fail("Failed to delete test resources.");
        }
    }

    public void testUserDefinedResourceQuery() {
        try {

            Resource r1 = new Resource();
            String r1Content = "this is r1 content";
            r1.setContent(r1Content.getBytes());
            r1.setDescription("production ready.");
            String r1Path = "/c1/r1";
            registry.put(r1Path, r1);

            Resource r2 = new Resource();
            String r2Content = "content for r2 :)";
            r2.setContent(r2Content);
            r2.setDescription("ready for production use.");
            String r2Path = "/c2/r2";
            registry.put(r2Path, r2);

            Resource r3 = new Resource();
            String r3Content = "content for r3 :)";
            r3.setContent(r3Content);
            r3.setDescription("only for government use.");
            String r3Path = "/c2/r3";
            registry.put(r3Path, r3);

            String sql1 = "SELECT PATH FROM ARTIFACTS WHERE DESCRIPTION LIKE ?";
            Resource q1 = new Resource();
            q1.setContent(sql1);
            q1.setMediaType(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
            q1.setProperty(RegistryConstants.RESULT_TYPE_PROPERTY_NAME,
                    RegistryConstants.RESOURCES_RESULT_TYPE);
            registry.put("/qs/q1", q1);

        } catch (RegistryException e) {
            fail("Valid put resource scenarios failed.");
        }

        try {
            Resource result1 = registry.executeQuery("/qs/q1", new String[] {"%production%"});

            assertTrue("Search with result type Resource should return a directory.",
                    result1.isDirectory());

            List matchingPaths = new ArrayList();
            String[] paths = (String[]) result1.getContent();
            for (int i = 0; i < paths.length; i++) {
                matchingPaths.add(paths[i]);
            }

            assertTrue("Path /c1/r1 should be in the results.", matchingPaths.contains("/c1/r1"));
            assertTrue("Path /c2/r2 should be in the results.", matchingPaths.contains("/c2/r2"));

        } catch (RegistryException e) {
            fail("Failed to execute query.");
        }
    }

    public void testUserDefinedRatingsQuery() {
        try {

            Resource r1 = new Resource();
            String r1Content = "this is r1 content";
            r1.setContent(r1Content.getBytes());
            r1.setDescription("production ready.");
            String r1Path = "/c3/r1";
            registry.put(r1Path, r1);

            Resource r2 = new Resource();
            String r2Content = "content for r2 :)";
            r2.setContent(r2Content);
            r2.setDescription("ready for production use.");
            String r2Path = "/c3/r2";
            registry.put(r2Path, r2);

            Resource r3 = new Resource();
            String r3Content = "content for r3 :)";
            r3.setContent(r3Content);
            r3.setDescription("only for government use.");
            String r3Path = "/c3/r3";
            registry.put(r3Path, r3);

            registry.rateResource("/c3/r1", 3);
            registry.rateResource("/c3/r2", 4);
            registry.rateResource("/c3/r3", 5);

            String sql1 = "SELECT R.R_ID FROM RATINGS R, ARTIFACTS A WHERE A.AID=R.AID AND A.DESCRIPTION LIKE ?";
            Resource q1 = new Resource();
            q1.setContent(sql1);
            q1.setMediaType(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
            q1.setProperty(RegistryConstants.RESULT_TYPE_PROPERTY_NAME,
                    RegistryConstants.RATINGS_RESULT_TYPE);
            registry.put("/qs/q2", q1);

        } catch (RegistryException e) {
            fail("Valid put resource scenarios failed.");
        }

        try {
            Resource result1 = registry.executeQuery("/qs/q2", new String[] {"%production%"});

            assertTrue("Search with result type Resource should return a directory.",
                    result1.isDirectory());

            String[] ratingPaths = (String[]) result1.getContent();
            assertEquals("There should be two match ratings.", ratingPaths.length, 2);

            Resource rating1 = registry.get(ratingPaths[0]);
            assertEquals("First matching rating should be 3",
                    rating1.getContent().toString(), "3");

            Resource rating2 = registry.get(ratingPaths[1]);
            assertEquals("First matching rating should be 3",
                    rating2.getContent().toString(), "4");

        } catch (RegistryException e) {
            fail("Failed to execute query.");
        }
    }

    public void testUserDefinedTagsQuery() {
        try {

            Resource r1 = new Resource();
            String r1Content = "this is r1 content";
            r1.setContent(r1Content.getBytes());
            r1.setDescription("production ready.");
            String r1Path = "/c3/r1";
            registry.put(r1Path, r1);

            Resource r2 = new Resource();
            String r2Content = "content for r2 :)";
            r2.setContent(r2Content);
            r2.setDescription("ready for production use.");
            String r2Path = "/c3/r2";
            registry.put(r2Path, r2);

            Resource r3 = new Resource();
            String r3Content = "content for r3 :)";
            r3.setContent(r3Content);
            r3.setDescription("only for government use.");
            String r3Path = "/c3/r3";
            registry.put(r3Path, r3);

            registry.applyTag("/c3/r1", "java");
            registry.applyTag("/c3/r2", "jsp");
            registry.applyTag("/c3/r3", "ajax");

            String sql1 = "SELECT T.TN_ID FROM TAGS T, ARTIFACTS A WHERE A.AID=T.AID AND A.DESCRIPTION LIKE ?";
            Resource q1 = new Resource();
            q1.setContent(sql1);
            q1.setMediaType(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
            q1.setProperty(RegistryConstants.RESULT_TYPE_PROPERTY_NAME,
                    RegistryConstants.TAGS_RESULT_TYPE);
            registry.put("/qs/q3", q1);

        } catch (RegistryException e) {
            fail("Valid put resource scenarios failed.");
        }

        try {
            Resource result1 = registry.executeQuery("/qs/q3", new String[] {"%production%"});

            assertTrue("Search with result type tags should return a directory.",
                    result1.isDirectory());

            String[] tagPaths = (String[]) result1.getContent();
            assertEquals("There should be two matching tags.", tagPaths.length, 2);

            Resource tag1 = registry.get(tagPaths[0]);
            assertEquals("First matching tag should be 'java'",
                    (String)tag1.getContent(), "java");

            Resource tag2 = registry.get(tagPaths[1]);
            assertEquals("First matching tag should be 'jsp'",
                    (String)tag2.getContent(), "jsp");

        } catch (RegistryException e) {
            fail("Failed to execute query.");
        }
    }

    public void testUserDefinedCommentsQuery() {
        try {

            Resource r1 = new Resource();
            String r1Content = "this is r1 content";
            r1.setContent(r1Content.getBytes());
            r1.setDescription("production ready.");
            String r1Path = "/c5/r1";
            registry.put(r1Path, r1);

            Resource r2 = new Resource();
            String r2Content = "content for r2 :)";
            r2.setContent(r2Content);
            r2.setDescription("ready for production use.");
            String r2Path = "/c5/r2";
            registry.put(r2Path, r2);

            Resource r3 = new Resource();
            String r3Content = "content for r3 :)";
            r3.setContent(r3Content);
            r3.setDescription("only for government use.");
            String r3Path = "/c5/r3";
            registry.put(r3Path, r3);

            registry.addComment("/c5/r1", new Comment("we have to change this file."));
            registry.addComment("/c5/r2", new Comment("replace this with a better one"));
            registry.addComment("/c5/r3", new Comment("import all dependencies for this resource"));

            String sql1 = "SELECT C.CM_ID FROM COMMENTS C, ARTIFACTS A WHERE A.AID=C.AID AND A.DESCRIPTION LIKE ?";
            Resource q1 = new Resource();
            q1.setContent(sql1);
            q1.setMediaType(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
            q1.setProperty(RegistryConstants.RESULT_TYPE_PROPERTY_NAME,
                    RegistryConstants.COMMENTS_RESULT_TYPE);
            registry.put("/qs/q4", q1);

        } catch (RegistryException e) {
            fail("Valid put resource scenarios failed.");
        }

        try {
            Resource result1 = registry.executeQuery("/qs/q4", new String[] {"%production%"});

            assertTrue("Search with result type Comment should return a collection.",
                    result1.isDirectory());

            String[] commentPaths = (String[]) result1.getContent();
            assertEquals("There should be two match comments.", commentPaths.length, 2);

            Resource c1 = registry.get(commentPaths[0]);
            assertEquals("First matching comment is incorrect",
                    (String) c1.getContent(), "we have to change this file.");

            Resource c2 = registry.get(commentPaths[1]);
            assertEquals("Second matching comment is incorrect",
                    (String) c2.getContent(), "replace this with a better one");

        } catch (RegistryException e) {
            fail("Failed to execute query.");
        }
    }

    public void testTagsAsResources() {
        //Resource r1 = new Resource();
        //String r1Content = "this is r1 content";
        //r1.setContent(r1Content.getBytes());
        //r1.setDescription("production ready.");
        //String r1Path = "/c1/r1";
        //
        //try {
        //    registry.put(r1Path, r1);
        //} catch (RegistryException e) {
        //    fail("Valid put scenario failed.");
        //}
        //
        //try {
        //    registry.applyTag("/c1/r1", "java");
        //    registry.applyTag("/c1/r1", "bookmarking");
        //    registry.applyTag("/c1/r1", "registry");
        //} catch (RegistryException e) {
        //    fail("Valid tagging scenario failed.");
        //}
        //
        //Resource tags = null;
        //try {
        //    tags = registry.get("/c1/r1?tags");
        //} catch (RegistryException e) {
        //    fail("Failed to get tags using URL.");
        //}
        //
        //String[] tagPaths = (String[]) tags.getContent();
        //List tagStrings = new ArrayList();
        //
        //for (int i = 0; i < tagPaths.length; i++) {
        //    try {
        //        Resource c1 = registry.get(tagPaths[i]);
        //        String tagString = (String) c1.getContent();
        //        tagStrings.add(tagString);
        //    } catch (RegistryException e) {
        //        fail("Failed to get comment using a path.");
        //    }
        //}
        //
        //assertTrue("Tags are not retrieved properly as resources.",
        //        tagStrings.contains("java"));
        //
        //assertTrue("Tags are not retrieved properly as resources.",
        //        tagStrings.contains("bookmarking"));
        //
        //assertTrue("Tags are not retrieved properly as resources.",
        //        tagStrings.contains("registry"));
        //
        //int a = 1;
    }

    public void testCommentsAsResources() {
        Resource r1 = new Resource();
        String r1Content = "this is r1 content";
        r1.setContent(r1Content.getBytes());
        r1.setDescription("production ready.");
        String r1Path = "/c1/r1";

        try {
            registry.put(r1Path, r1);
        } catch (RegistryException e) {
            fail("Valid put scenario failed.");
        }

        try {
            registry.addComment("/c1/r1", new Comment("this is used to test comments."));
            registry.addComment("/c1/r1", new Comment("dummy resource."));
            registry.addComment("/c1/r1", new Comment("simple test resource."));
        } catch (RegistryException e) {
            fail("Valid tagging scenario failed.");
        }

        Resource comments = null;
        try {
            comments = registry.get("/c1/r1?comments");
        } catch (RegistryException e) {
            fail("Failed to get comments using URL.");
        }

        String[] commentPaths = (String[]) comments.getContent();
        List commentStrings = new ArrayList();

        for (int i = 0; i < commentPaths.length; i++) {
            try {
                Resource c1 = registry.get(commentPaths[i]);
                String commentString = (String) c1.getContent();
                commentStrings.add(commentString);
            } catch (RegistryException e) {
                fail("Failed to get comment using a path.");
            }
        }

        assertTrue("Comments are not retrieved properly as resources.",
                commentStrings.contains("this is used to test comments."));

        assertTrue("Comments are not retrieved properly as resources.",
                commentStrings.contains("dummy resource."));

        assertTrue("Comments are not retrieved properly as resources.",
                commentStrings.contains("simple test resource."));
    }

    public void testRatingsAsResources() {
        Resource r5 = new Resource();
        String r5Content = "this is r5 content";
        r5.setContent(r5Content.getBytes());
        r5.setDescription("production ready.");
        String r5Path = "/c1/r5";

        try {
            registry.put(r5Path, r5);
        } catch (RegistryException e) {
            fail("Valid put scenario failed.");
        }

        try {
            registry.rateResource("/c1/r5", 3);
        } catch (RegistryException e) {
            fail("Valid tagging scenario failed.");
        }

        Resource ratings = null;
        try {
            ratings = registry.get("/c1/r5;ratings");
        } catch (RegistryException e) {
            fail("Failed to get ratings using URL.");
        }

        String[] ratingPaths = (String[]) ratings.getContent();

        int rating = 0;
        try {
            Resource c1 = registry.get(ratingPaths[0]);

            Object o = c1.getContent();
            if(o instanceof Integer) {
                rating = ((Integer) o).intValue();
            } else {
                rating = Integer.parseInt(o.toString());
            }

        } catch (RegistryException e) {
            fail("Failed to get rating using a path.");
        }

        assertEquals("Ratings are not retrieved properly as resources.", rating, 3);
    }

    public void testLogs() throws RegistryException {
        String r1Content = "this is the r200 content.";
        Resource r1 = new Resource();
        r1.setContent(r1Content.getBytes());

        try {
            registry.put("/r200", r1);
        } catch (RegistryException e) {
            fail("Couldn't put a content resource in to path /r200");
        }

        registry.rateResource("/r200", 5);

        LogEntry[] logs = registry.getLogs("/r200", -1, null, null, null);

        LogEntry l1 = logs[0];
        assertEquals("Log for adding /r200 is not added properly.", LogEntry.UPDATE, l1.getAction());
        assertEquals("Log for adding /r200 is not added properly.", "/r200", l1.getResourcePath());

        LogEntry l2 = logs[1];
        assertEquals("Log for rating /r200 is not added properly.", LogEntry.RATING, l2.getAction());
        assertEquals("Log for rating /r200 is not added properly.", "5", l2.getActionData());
    }

    public void testResourceDelete() throws RegistryException{
        String content1 = "Content1";
        Resource r1 = new Resource();
        r1.setContent(content1);
        try {
            //Adding a resource
            registry.put("/wso2/wsas/v1/r1", r1);

        } catch (RegistryException e) {
            fail("Couldn't put a content resource in to path /wso2/wsas/v1/r1");
        }

        // Adding a dummy resorce
        Resource v2 = new Resource();
        registry.put("/wso2/wsas/v2", v2);

        //getting the resource
        Resource r2=   registry.get("/wso2/wsas/v1");

        //chcek whether the content is corrcet
        assertEquals("/wso2/wsas/v1/r1",((String[])r2.getContent())[0]);
        Resource wsas = registry.get("/wso2/wsas");
        String wasaContent [] = (String[]) wsas.getContent();
        assertNotNull(wasaContent);
        assertEquals(2,wasaContent.length);

        registry.delete("/wso2/wsas/v1");

        String content2 ="Content2";
        Resource resourceContent2 = new Resource();
        resourceContent2.setContent(content2);
        registry.put("/wso2/wsas/v1/r2",resourceContent2);

        wsas = registry.get("/wso2/wsas");
        wasaContent = (String[]) wsas.getContent();
        assertNotNull(wasaContent);
        assertEquals(2,wasaContent.length);

        r2=   registry.get("/wso2/wsas/v1");
        //chcek whether the content is corrcet
        assertEquals("/wso2/wsas/v1/r2",((String[])r2.getContent())[0]);

        registry.restoreVersion("/wso2/wsas?v=2");
         r2=   registry.get("/wso2/wsas/v1");
        //chcek whether the content is corrcet
        assertEquals("/wso2/wsas/v1/r1",((String[])r2.getContent())[0]);

    }

    public void testCombinedScenario() throws RegistryException {
        // put a content resource in to the root
        String r1Content = "this is the r1 content.";
        Resource r1 = new Resource();
        r1.setContent(r1Content.getBytes());
        try {
            registry.put("/r11", r1);
        } catch (RegistryException e) {
            fail("Couldn't put a content resource in to path /r11");
        }

        // put a collection in to the root

        Resource c1 = new Resource();
        c1.setDirectory(true);

        try {
            registry.put("/c1", c1);
        } catch (RegistryException e) {
            fail("Couldn't put a collection artifact in to path /c1");
        }

        // put a content artifact in to /c1/r2

        String r2Content = "this is r2 content";
        Resource r2 = new Resource();
        r2.setContent(r2Content.getBytes());

        try {
            registry.put("/c1/r2", r2);
        } catch (RegistryException e) {
            fail("Couldn't put new content resource in to existing collection /c1");
        }

        // put a content artifact in to non-existing collection

        String r3Content = "this is r3 content";
        Resource r3 = new Resource();
        Properties props = new Properties();
        props.put("Reviewer", "Deepal");
        props.put("Testedon", "Axis2");
        r3.setProperties(props);
        r3.setContent(r3Content.getBytes());

        try {
            registry.put("/c2/r3", r3);
        } catch (RegistryException e) {
            fail("Couldn't put new content resource in to non-existing collection /c2");
        }

        // put c2/r4
        String r4Content = "this is r4 content";
        Resource r4 = new Resource();
        r4.setContent(r4Content.getBytes());

        registry.put("/c2/r4", r4);


        // do some taggings

        try {
            registry.applyTag("/c2/r4", "ui");
            registry.applyTag("/c2/r4", "swing");
            registry.applyTag("/r1", "test");
            registry.applyTag("/r1", "Java");
            registry.applyTag("/c1", "test");
            registry.applyTag("/c1/r2", "struts");
            registry.applyTag("/c1/r2", "ui");
            registry.applyTag("/c1/r2", "web");
            registry.applyTag("/c1/r2", "web2");
            registry.applyTag("/c1/r2", "web3");

        } catch (Exception e) {
            fail("Tagging artifacts failed.");
        }

        // add some comments

        try {
            registry.addComment("/r1", new Comment("this is a test artifact."));
            registry.addComment("/c1", new Comment("I don't agree with having this collection."));
            registry.addComment("/c1/r2", new Comment("Is this a collection ;)"));
            registry.addComment("/c2", new Comment("I can understand the purpose of this. Thanks."));
        } catch (RegistryException e) {
            fail("Valid commenting scenario failed.");
        }

        // rate some artifacts

        try {
            registry.rateResource("/r11", 3);
            registry.rateResource("/r11", 4);
            registry.rateResource("/r11", 5);
            registry.rateResource("/c1", 2);
            registry.rateResource("/c1/r2", 5);
            registry.rateResource("/c2/r4", 1);
            registry.rateResource("/c2/r4", 2);

        } catch (RegistryException e) {
            fail("Valid rating scenario failed.");
        }

        registry.delete("/r11");
        registry.delete("/c1");
        registry.delete("/c2");
    }

    private boolean containsString(String[] array, String value) {

        boolean found = false;
        for (int i = 0; i < array.length; i++) {
            if (array[i].startsWith(value)) {
                found = true;
                break;
            }
        }

        return found;
    }
}
