/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.secure;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.servlet.RegistryRealmFactory;
import org.wso2.registry.jdbc.JDBCRegistry;
import org.wso2.usermanager.Realm;

public class SecureRegistryTest extends TestCase {

    private static Registry registry = null;
    private static Realm realm = null;

    public void setUp() {
        try {
            if (registry == null) {
                realm = RegistryRealmFactory.createInMemoryRegistryRealm();
                JDBCRegistry jdbcRegistry = new JDBCRegistry();
                jdbcRegistry.init(realm);
                registry = jdbcRegistry;
            }
        } catch (RegistryException e) {
            fail("Failed to initialize the registry.");
        }
    }

    public void testUserBasedAuthorization() throws RegistryException {

        SecureRegistry adminRegistry =
                new SecureRegistry(RegistryConstants.ADMIN_USER, "admin", registry, realm);
        RegistryUserManager adminUserManager =
                new RegistryUserManager(realm, RegistryConstants.ADMIN_USER);

        String r1Content = "R1";
        Resource r1 = new Resource();
        r1.setContent(r1Content.getBytes());

        try {
            adminRegistry.put("/r1", r1);
        } catch (RegistryException e) {
            fail("Valid put scenario failed.");
        }

        adminUserManager.addUser("chathura", "cce");
        adminUserManager.addUser("upul", "u" );

        adminUserManager.authorizeUser("chathura", "/r1", ActionConstants.PUT);

        SecureRegistry chathuraRegistry =
                new SecureRegistry("chathura", "cce", registry, realm);

        try {
            chathuraRegistry.get("/r1");
        } catch (AuthorizationFailedException e) {
            fail("Could not get authorized resource.");
        }

        SecureRegistry upulRegistry =
                new SecureRegistry("upul", "u", registry, realm);

        boolean failed = false;
        try {
            upulRegistry.put("/r1", r1);
        } catch (AuthorizationFailedException e) {
            failed = true;
        }

        assertTrue("Allowed to get unauthorized resource.", failed);
    }

    public void testRoleBasedAuthorization() throws RegistryException {

        SecureRegistry adminRegistry =
                new SecureRegistry(RegistryConstants.ADMIN_USER, "admin", registry, realm);
        RegistryUserManager adminUserManager =
                new RegistryUserManager(realm, RegistryConstants.ADMIN_USER);

        String r1Content = "R1";
        Resource r1 = new Resource();
        r1.setContent(r1Content.getBytes());

        try {
            adminRegistry.put("/r1", r1);
        } catch (RegistryException e) {
            fail("Valid put scenario failed.");
        }

        adminUserManager.addRole("registryTeam");
        adminUserManager.authorizeRole("registryTeam", "/r1", ActionConstants.PUT);

        adminUserManager.addUser("chathurax", "cce");
        adminUserManager.addUserToRole("chathurax", "registryTeam");

        adminUserManager.addUser("upulx", "u");

        SecureRegistry chathuraRegistry =
                new SecureRegistry("chathurax", "cce", registry, realm);

        try {
            chathuraRegistry.get("/r1");
        } catch (AuthorizationFailedException e) {
            fail("Could not get authorized resource.");
        }

        SecureRegistry upulRegistry =
                new SecureRegistry("upulx", "u", registry, realm);

        boolean failed = false;
        try {
            upulRegistry.put("/r1", r1);
        } catch (AuthorizationFailedException e) {
            failed = true;
        }

        assertTrue("Allowed to get unauthorized resource.", failed);
    }

    //public void putResourceTest() {
    //
    //    try {
    //        RegistryRealmFactory realmFactory = new RegistryRealmFactory();
    //        Realm realm = realmFactory.newRegistryRealm();
    //        RegistryUserManager userManager = new RegistryUserManager(realm);
    //
    //        SecureRegistry adminRegistry = new SecureRegistry(
    //                RegistryConstants.ADMIN_USER, "admin", registry, realm);
    //
    //        //userManager.addUser("chathura", "cce123", RegistryConstants.ADMIN_USER);
    //        SecureRegistry chathuraRegistry = new SecureRegistry(
    //                "chathura", "cce123", registry, realm);
    //
    //        Resource a1 = new Resource();
    //        a1.setContent("test".getBytes());
    //        adminRegistry.put("/d1/d2/d3/r1", a1);
    //
    //        Resource a2 = new Resource();
    //        a2.setContent("chathura's content".getBytes());
    //
    //        boolean failed = false;
    //        try {
    //            chathuraRegistry.put("/d1/d2/d4/r2", a2);
    //        } catch (RegistryException e) {
    //            failed = true;
    //        }
    //        assertTrue("Allowed put a resource to an unauthorized path.", failed);
    //
    //        userManager.authorizeUser(
    //                "chathura", "/d1/d2", ActionConstants.PUT, RegistryConstants.ADMIN_USER);
    //
    //        try {
    //            chathuraRegistry.put("/d1/d2/d4/r2", a2);
    //        } catch (RegistryException e) {
    //            fail("Denied to put a resource to an authorized path.");
    //        }
    //
    //    } catch (RegistryException e) {
    //        e.printStackTrace();
    //    }
    //}

    public void testAdvancedSearch() {

        try {

            String resourcePath = "";
            String authorName = "";
            String updaterName = "xSamitha";
            String tags = "";
            String commentWords = "";
            String content;
            String resultType;

            // create the sql

            boolean queryStarted = false;

            String pathQuery = "";
            if (resourcePath != null && resourcePath.length() != 0) {
                pathQuery = " WHERE R.PATH='" + resourcePath + "'";
                queryStarted = true;
            }

            String authorQuery = "";
            if (authorName != null && authorName.length() != 0) {
                if (queryStarted) {
                    authorQuery = " AND R.AUTHOR='" + authorName + "'";
                } else {
                    authorQuery = " WHERE R.AUTHOR='" + authorName + "'";
                    queryStarted = true;
                }
            }

            //String sql = "SELECT A.AID FROM ARTIFACTS A, VERSIONS V, TAGS T, COMMENTS C WHERE A.AID=V.AID AND V.VN IN (SELECT MAX(V2.VN) FROM VERSIONS V2 WHERE A.AID=V2.AID) AND A.AID=T.AID AND A.AID=C.AID AND A.PATH='c1200/r4'";
            String fromVersions = "";
            String versionQuery = "";
            if (updaterName != null && updaterName.length() != 0) {
                fromVersions = ", VERSIONS V";
                if (queryStarted) {
                    versionQuery = " AND V.AID=R.AID AND V.VN IN (SELECT MAX(V2.VN) FROM VERSIONS V2 WHERE R.AID=V2.AID) AND V.AUTHOR='" + updaterName + "'";
                } else {
                    versionQuery = " WHERE V.AID=R.AID AND V.VN IN (SELECT MAX(V2.VN) FROM VERSIONS V2 WHERE R.AID=V2.AID) AND V.AUTHOR='" + updaterName + "'";
                }
            }

            String fromComments = "";
            String commentsQuery = "";
            if (commentWords != null && commentWords.length() != 0) {
                fromComments = ", COMMENTS C";
                if (queryStarted) {
                    commentsQuery = " AND C.AID=R.AID AND C.COMMENT_TEXT LIKE '" + commentWords + "'";
                } else {
                    commentsQuery = " WHERE C.AID=R.AID AND C.COMMENT_TEXT LIKE '%" + commentWords + "%'";
                    queryStarted = true;
                }
            }

            String fromTags = "";
            String tagsQuery = "";
            if (tags != null && tags.length() != 0) {
                String[] tag = tags.split(" ");
                if (tag.length > 0) {
                    fromTags = ", TAGS T";

                    // todo: complete with OR tags
                    if (queryStarted) {
                        tagsQuery = " AND T.AID=R.AID AND";
                    } else {
                        tagsQuery = " WHERE T.AID=R.AID AND";
                        queryStarted = true;
                    }

                    tagsQuery = tagsQuery + " (T.TAG_NAME='" + tag[0] + "'";
                    for (int i = 1; i < tag.length; i++) {
                        tagsQuery = tagsQuery + " OR T.TAG_NAME='" + tag[i] + "'";
                    }
                    tagsQuery = tagsQuery + ")";
                }
            }

            String sql = "SELECT R.PATH FROM ARTIFACTS R" + fromVersions + fromComments + fromTags + pathQuery + authorQuery + versionQuery + commentsQuery + tagsQuery;

            Resource q1 = new Resource();
            q1.setContent(sql);
            q1.setMediaType(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
            q1.setProperty(RegistryConstants.RESULT_TYPE_PROPERTY_NAME,
                    RegistryConstants.RESOURCES_RESULT_TYPE);
            registry.put("/advanced/q1", q1);

            SecureRegistry adminRegistry =
                    new SecureRegistry(RegistryConstants.ADMIN_USER, "admin", registry, realm);
            RegistryUserManager adminUserManager =
                    new RegistryUserManager(realm, RegistryConstants.ADMIN_USER);

            adminUserManager.addUser("xChathura", "cce");
            adminUserManager.addUserToRole("xChathura", RegistryConstants.ADMIN_ROLE);
            SecureRegistry chathuraRegistry =
                    new SecureRegistry("xChathura", "cce", registry, realm);

            adminUserManager.addUser("xSamitha", "swe");
            adminUserManager.addUserToRole("xSamitha", RegistryConstants.ADMIN_ROLE);
            SecureRegistry samithaRegistry =
                    new SecureRegistry("xSamitha", "swe", registry, realm);

            String r1Content = "R1";
            Resource r1 = new Resource();
            r1.setContent(r1Content.getBytes());
            adminRegistry.put("/c1200/r1", r1);

            String r2Content = "R2";
            Resource r2 = new Resource();
            r2.setContent(r2Content.getBytes());
            adminRegistry.put("/c1200/r2", r2);

            String r3Content = "R3";
            Resource r3 = new Resource();
            r3.setContent(r3Content.getBytes());
            chathuraRegistry.put("/c1200/r3", r3);

            String r4Content = "R4";
            Resource r4 = new Resource();
            r4.setContent(r4Content.getBytes());
            samithaRegistry.put("/c1200/r2", r4);

            adminRegistry.applyTag("/c1200/r1", "java jsp software");
            adminRegistry.applyTag("/c1200/r2", "java hibernate");
            adminRegistry.applyTag("/c1200/r3", "c windows");

            chathuraRegistry.addComment("/c1200/r2", new Comment("this is some comment on this resource."));

            //String r1v2Content = "R2";
            //Resource r1v2 = new Resource();
            //r1v2.setContent(r1v2Content.getBytes());
            //chathuraRegistry.put("/c1200/r4", r1v2);

            //String sql = "SELECT A.AID FROM ARTIFACTS A, VERSIONS V, TAGS T, COMMENTS C WHERE A.AID=V.AID AND V.VN IN (SELECT MAX(V2.VN) FROM VERSIONS V2 WHERE A.AID=V2.AID) AND A.AID=T.AID AND A.AID=C.AID AND A.PATH='c1200/r4'";
            //String sql = "SELECT A.PATH FROM ARTIFACTS A WHERE A.PATH='/c1200/r4'";
            
            Resource q1Results = registry.executeQuery("/advanced/q1", new String[] {} );
            int a = 1;

        } catch (RegistryException e) {
            e.printStackTrace();
        }


    }
}
